'use strict';

import fs from 'fs';
import http from 'http';
import onvif from 'node-onvif';
import websocket from 'websocket';
const WebSocketServer = websocket.server;
var port = 8880;

(function main() {
	var http_server = http.createServer(httpServerRequest);
	http_server.listen(port, function () {
		console.log('Listening on port ' + port);
	});
	var wsserver = new WebSocketServer({
		httpServer: http_server,
	});
	wsserver.on('request', wsServerRequest);
})();

function httpServerRequest(req, res) {
	var path = req.url.replace(/\?.*$/, '');
	if (path.match(/\.{2,}/) || path.match(/[^a-zA-Z\d\_\-\.\/]/)) {
		httpServerResponse404(req.url, res);
		return;
	}
	if (path === '/') {
		path = '/index.html';
	}
	var fpath = './html' + path;
	fs.readFile(fpath, 'utf-8', function (err, data) {
		if (err) {
			httpServerResponse404(req.url, res);
			return;
		} else {
			var ctype = getContentType(fpath);
			res.writeHead(200, { 'Content-Type': ctype });
			res.write(data);
			res.end();
			console.log('HTTP : 200 OK : ' + req.url);
		}
	});
}

function getContentType(fpath) {
	var ext = fpath.split('.').pop().toLowerCase();
	if (ext.match(/^(html|htm)$/)) {
		return 'text/html';
	} else if (ext.match(/^(jpeg|jpg)$/)) {
		return 'image/jpeg';
	} else if (ext.match(/^(png|gif)$/)) {
		return 'image/' + ext;
	} else if (ext === 'css') {
		return 'text/css';
	} else if (ext === 'js') {
		return 'text/javascript';
	} else if (ext === 'woff2') {
		return 'application/font-woff';
	} else if (ext === 'woff') {
		return 'application/font-woff';
	} else if (ext === 'ttf') {
		return 'application/font-ttf';
	} else if (ext === 'svg') {
		return 'image/svg+xml';
	} else if (ext === 'eot') {
		return 'application/vnd.ms-fontobject';
	} else if (ext === 'oft') {
		return 'application/x-font-otf';
	} else {
		return 'application/octet-stream';
	}
}

function httpServerResponse404(url, res) {
	res.writeHead(404, { 'Content-Type': 'text/plain' });
	res.write('404 Not Found: ' + url);
	res.end();
	console.log('HTTP : 404 Not Found : ' + url);
}

var client_list = [];

function wsServerRequest(request) {
	var conn = request.accept(null, request.origin);
	conn.on('message', function (message) {
		if (message.type !== 'utf8') {
			return;
		}
		var data = JSON.parse(message.utf8Data);
		var method = data['method'];
		var params = data['params'];
		if (method === 'startDiscovery') {
			startDiscovery(conn);
		} else if (method === 'connect') {
			connect(conn, params);
		} else if (method === 'fetchSnapshot') {
			fetchSnapshot(conn, params);
		} else if (method === 'ptzMove') {
			ptzMove(conn, params);
		} else if (method === 'ptzStop') {
			ptzStop(conn, params);
		} else if (method === 'ptzHome') {
			ptzHome(conn, params);
		}
	});

	conn.on('close', function (message) {});
	conn.on('error', function (error) {
		console.log(error);
	});
}

var devices = {};
const cameras = [
	{
		name: 'Camera 1',
		slug: 'camera-1',
		ip: 'sanbongthanhcong.cameraddns.net',
		port: 86,
		username: 'admin2',
		password: 'abcd1234',
	},
];

function startDiscovery(conn) {
	devices = {};
	var devs = {};
	const _deviceLists = cameras;
	_deviceLists.forEach(device => {
		let odevice = new onvif.OnvifDevice({
			xaddr: `http://${device.ip}:${device.port}/onvif/device_service`,
		});
		let addr = odevice.address;
		odevice.name = device.name;
		devices[addr + ':' + device.port] = odevice;
		devs[device.slug] = {
			name: device.name,
			address: device.ip + ':' + device.port,
		};
	});
	let res = { id: 'startDiscovery', result: devs };
	conn.send(JSON.stringify(res));
}

function connect(conn, params) {
	var res = { id: 'connect' };
	res['result'] = {
		Manufacturer: 'Hikvision',
		Model: devices[params.address].name,
		FirmwareVersion: '',
		SerialNumber: '',
		HardwareId: '',
	};
	conn.send(JSON.stringify(res));

	/*var device = devices[params.address];
	if (!device) {
		var res = { id: 'connect', error: 'The specified device is not found: ' + params.address };
		conn.send(JSON.stringify(res));
		return;
	}
	if (params.user) {
		device.setAuth(params.user, params.pass);
	}
	device.init((error, result) => {
		var res = { id: 'connect' };
		console.log(result);
		if (error) {
			res['error'] = error.toString();
		} else {
			res['result'] = result;
		}
		conn.send(JSON.stringify(res));
	});*/
}

// For Debug --------------------------------------------
//var total_size = 0;
//var start_time = 0;
//var total_frame = 0;
// ------------------------------------------------------
import DigestClient from 'digest-fetch';
function fetchSnapshot(conn, params) {
	// For Debug --------------------------------------------
	//if(start_time === 0) {
	//	start_time = Date.now();
	//}
	// ------------------------------------------------------
	/*var device = devices[params.address];
	if (!device) {
		var res = { id: 'fetchSnapshot', error: 'The specified device is not found: ' + params.address };
		conn.send(JSON.stringify(res));
		return;
	}*/
	/*device.fetchSnapshot((error, result) => {
		var res = { id: 'fetchSnapshot' };
		if (error) {
			res['error'] = error.toString();
		} else {
			var ct = result['headers']['content-type'];
			var buffer = result['body'];
			var b64 = buffer.toString('base64');
			var uri = 'data:' + ct + ';base64,' + b64;
			res['result'] = uri;
			
		}
		conn.send(JSON.stringify(res));
	});*/
	const client = new DigestClient('admin2', 'abcd1234');
	client.fetch('http://sanbongthanhcong.cameraddns.net:86/onvif-http/snapshot?Profile_1').then(async resp => {
		var res = { id: 'fetchSnapshot' };
		var ct = resp.headers.get('content-type');
		var blob = await resp.blob();
		var stream = blob.stream();
		var buffer = [];

		for await (const chunk of stream) {
			buffer.push(chunk);
		}
		var b64 = Buffer.concat(buffer).toString('base64');
		var uri = 'data:' + ct + ';base64,' + b64;
		res['result'] = uri;
		conn.send(JSON.stringify(res));
	});
}

function ptzMove(conn, params) {
	var device = devices[params.address];
	if (!device) {
		var res = { id: 'ptzMove', error: 'The specified device is not found: ' + params.address };
		conn.send(JSON.stringify(res));
		return;
	}
	device.ptzMove(params, error => {
		var res = { id: 'ptzMove' };
		if (error) {
			res['error'] = error.toString();
		} else {
			res['result'] = true;
		}
		conn.send(JSON.stringify(res));
	});
}

function ptzStop(conn, params) {
	var device = devices[params.address];
	if (!device) {
		var res = { id: 'ptzStop', error: 'The specified device is not found: ' + params.address };
		conn.send(JSON.stringify(res));
		return;
	}
	device.ptzStop(error => {
		var res = { id: 'ptzStop' };
		if (error) {
			res['error'] = error.toString();
		} else {
			res['result'] = true;
		}
		conn.send(JSON.stringify(res));
	});
}

function ptzHome(conn, params) {
	var device = devices[params.address];
	if (!device) {
		var res = { id: 'ptzMove', error: 'The specified device is not found: ' + params.address };
		conn.send(JSON.stringify(res));
		return;
	}
	if (!device.services.ptz) {
		var res = { id: 'ptzHome', error: 'The specified device does not support PTZ.' };
		conn.send(JSON.stringify(res));
		return;
	}

	var ptz = device.services.ptz;
	var profile = device.getCurrentProfile();
	var params = {
		ProfileToken: profile['token'],
		Speed: 1,
	};
	ptz.gotoHomePosition(params, (error, result) => {
		var res = { id: 'ptzMove' };
		if (error) {
			res['error'] = error.toString();
		} else {
			res['result'] = true;
		}
		conn.send(JSON.stringify(res));
	});
}
